#ifndef MEM_H
#define MEM_H

// Page
typedef u32int page_t;

// Page table
typedef page_t* ptbl_t;

// Page directory / "map"
typedef struct map {
	u32int *pdir;	// Physical addresses of tables
	ptbl_t *virt;	// Virtual addresses of tables
} map_t;

// note - one pool is 4 MB (1024 entries) of pages/frames
typedef struct pool {
	u32int word[32];
	u16int first;	// First free bit in pool
	u16int total;	// Total free bits in pool
	u16int setup;	// Magic number that verifies setup
	u16int upper;	// Highest actual bit in pool
} pool_t;

/****** SYSTEM CALLS *****/
u32int mmap(u32int base, u32int flags);	// Map memory to a page
u32int umap(u32int base);				// Unmap memory from a pagepage_t page_fmt(u32int base, u16int flags) {
u32int rmap(u32int base, u32int src);	// Move a mapped frame

/***** PRIVILEDGED SYSTEM CALLS *****/
u32int give(u32int base, u32int dest, u32int task);	// Grant a wormhole
u32int grab(u32int base, u32int src, u32int task);	// Get a wormhole by force
u32int phys(u32int base, u32int task);				// Get physical address

/***** ALLOC.C *****/
//u32int page_pool[31][32];
pool_t *ppool;
u16int total[31];
u32int proto_base;
// These are PAGE ALIGNED ALLOCATORS ONLY!
void *kmalloc(u32int size);	// Allocates one page
void kfree(void *addr);		// Frees one page

/***** POOL.C *****/
pool_t *pool_new(u32int num);				// Allocate a new pool
u32int pool_alloc(pool_t *pool);			// Allocate from a pool
u32int pool_free(pool_t *pool, u32int pos);	// Free back to a pool

/***** MAP.C *****/
map_t kmap;
map_t *map_alloc(map_t *map);							// Allocates a new map
map_t *map_free(map_t *map);							// Frees a map (does not clean)
map_t *map_clean(map_t *map, map_t *skip);				// Cleans a map (frees all memory)
map_t *map_clone(map_t *dest, map_t *src, map_t *link);	// Clones a map, and links
map_t *map_load(map_t *map);							// Activates a new map

/***** PAGE.C *****/
char mem_setup;
page_t page_touch(map_t *map, u32int page);				// Makes sure a page exists
page_t page_set(map_t *map, u32int page, page_t value);	// Sets the value of a page
page_t page_get(map_t *map, u32int page);				// Returns the value of a page
u32int phys_of(map_t *map, void *addr);					// Gets the physical address of a pointer
#define PF_MASK 0x0C67									// Page flags that can be used
#define page_fmt(base,flags) ((base&0xFFFFF000)|(flags&PF_MASK))
#define page_ufmt(page) (page&0xFFFFF000)

/***** FRAME.C ******/
pool_t *fpool;
u32int frame_new();				// Allocates a new frame
void frame_free(u32int addr);	// Frees a frame

/***** FLAGS *****/

// Normal page flags
#define PF_PRES 0x1		// Is present
#define PF_RW   0x2		// Is writeable
#define PF_USER 0x4		// Is user-mode
#define PF_DIRT 0x20	// Is dirty
#define PF_ACCS 0x40	// Has been accesses

// Special page flags
#define PF_HOLE 0x200	// Is a wormhole rented from another process
#define PF_LINK 0x400	// Is a wormhole granted to another process
#define PF_SWAP 0x800	// Is swapped out

#endif /*MEM_H*/
