; Multiboot stuff
MODULEALIGN equ  1<<0
MEMINFO     equ  1<<1
FLAGS       equ  MODULEALIGN | MEMINFO
MAGIC       equ  0x1BADB002
CHECKSUM    equ  -(MAGIC + FLAGS)

section .bss

STACKSIZE equ 0x2000
init_stack:
	resd STACKSIZE >> 2

section .pdata

; Initial kernel address space
global init_kmap
align 0x1000
init_kmap:
    dd 0x00000083	; Identity map first 4 MB
	times 991 dd 0	; Fill until 0xF8000000
	dd 0x00000083	; Map first 4 MB again in higher mem
	times 31 dd 0	; Fill remainder of map

section .data

gdt:
align 4
	dd 0x00000000, 0x00000000
	dd 0x0000FFFF, 0x00CF9A00
	dd 0x0000FFFF, 0x00CF9200
	dd 0x0000FFFF, 0x00CFFA00
	dd 0x0000FFFF, 0x00CFF200

gdt_ptr:
align 4
	dw 0x0027	; 39 bytes limit
	dd gdt - 0xF8000000	; Points to physical GDT

section .text

; Multiboot header
MultiBootHeader:
align 4
    dd MAGIC
    dd FLAGS
    dd CHECKSUM

extern init
global start
start:
	mov ecx, gdt_ptr - 0xF8000000	; Load (real) GDT pointer
	lgdt [ecx]			; Load new GDT
	mov ecx, 0x10		; Load all kernel data segments
	mov ds, cx
	mov es, cx
	mov fs, cx
	mov gs, cx
	mov ss, cx
	mov ecx, init_kmap - 0xF8000000	; Get physical address of the kernel address space
	mov cr3, ecx	; Load address into CR3
	mov ecx, cr4
	mov edx, cr0
	or ecx, 0x00000010	; Set 4MB page flag
	or edx, 0x80000000	; Set paging flag
	mov cr4, ecx		; Return to CR4 (make 4MB pgs)
	mov cr0, edx		; Return to CR0 (start paging)
	jmp 0x08:.upper		; Jump to the higher half (in new code segment)

.upper:
	mov esp, (init_stack + STACKSIZE)	; Setup init stack
	mov ebp, (init_stack + STACKSIZE)	; and base pointer
	push eax	; Push multiboot magic number
	add ebx, 0xF8000000	; Make pointer virtual
	push ebx	; Push multiboot pointer
	call init
	hlt			; Halt for now 
	
global redo_paging
redo_paging :
	mov eax, [esp+4]
	lea ecx, [.lower-0xF8000000]
	jmp ecx

.lower:
	mov ecx, cr0
	and ecx, 0x7FFFFFFF
	mov cr0, ecx
	mov ecx, cr4
	and ecx, 0xFFFFFFEF
	mov cr4, ecx
	mov cr3, eax
    mov ecx, cr0
    or ecx, 0x80000000
    mov cr0, ecx
	ret
